@@ -18,7 +18,6 @@
 import sys, gi
 
 gi.require_version('Gtk', '3.0')
-
 from gi.repository import Gtk, Gio, GLib, Gdk
 
 from .window import DrawingWindow
@@ -37,11 +36,13 @@ class Application(Gtk.Application):
 	shortcuts_window = None
 	prefs_window = None
 
+	# INITIALIZATION ###########################################################
+
 	def __init__(self, version):
 		super().__init__(application_id=APP_ID,
 		                 flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE)
 
-		GLib.set_application_name('Drawing')
+		GLib.set_application_name('Drawing') # XXX drawing ?
 		GLib.set_prgname(APP_ID)
 		self.version = version
 		self.git_url = 'https://github.com/maoschanz/drawing'
@@ -61,17 +62,19 @@ class Application(Gtk.Application):
 		                     GLib.OptionArg.NONE, _("Open a new window"), None)
 		self.add_main_option('new-tab', b't', GLib.OptionFlags.NONE,
 		                     GLib.OptionArg.NONE, _("Open a new tab"), None)
+		self.add_main_option('edit-clipboard', b'c', GLib.OptionFlags.NONE,
+		             GLib.OptionArg.NONE, _("Edit the clipboard content"), None)
+		# TODO options pour le screenshot ?
 
 		icon_theme = Gtk.IconTheme.get_default()
 		icon_theme.add_resource_path('/com/github/maoschanz/drawing/icons')
 		icon_theme.add_resource_path('/com/github/maoschanz/drawing/tools/icons')
 
-########
+		self.connect('window-removed', self.update_windows_menu_section)
 
 	def on_startup(self, *args):
-		"""Add app-wide menus and actions, and all accels."""
+		"""Called only once, add app-wide menus and actions, and all accels."""
 		self.build_actions()
-		self.add_accels()
 		builder = Gtk.Builder.new_from_resource( \
 		                        '/com/github/maoschanz/drawing/ui/app-menus.ui')
 		menubar_model = builder.get_object('menu-bar')
@@ -80,87 +83,62 @@ class Application(Gtk.Application):
 			appmenu_model = builder.get_object('app-menu')
 			self.set_app_menu(appmenu_model)
 
-	def is_beta(self):
-		"""Tells is the app version is even or odd, odd versions being considered
-		as unstable versions. This affects available options and the style of
-		the headerbar."""
-		version_array = self.version.split('.')
-		if int(version_array[1]) * 5 == 5:
-			return True
-		else:
-			return False
-
-	def add_action_simple(self, action_name, callback):
-		action = Gio.SimpleAction.new(action_name, None)
-		action.connect('activate', callback)
-		self.add_action(action)
-
-	def add_action_boolean(self, action_name, default, callback):
-		action = Gio.SimpleAction().new_stateful(action_name, None, \
-			GLib.Variant.new_boolean(default))
-		action.connect('change-state', callback)
-		self.add_action(action)
-
 	def build_actions(self):
 		"""Add app-wide actions."""
-		self.add_action_simple('new_window', self.on_new_window_activate)
-		self.add_action_simple('settings', self.on_prefs_activate)
+		self.add_action_simple('new_window', self.on_new_window, ['<Ctrl>n'])
+		self.add_action_simple('settings', self.on_prefs, None)
 		if self.is_beta():
-			self.add_action_simple('report_bug', self.on_report_activate)
-		self.add_action_simple('shortcuts', self.on_shortcuts_activate)
-		self.add_action_simple('help', self.on_help_activate)
-		self.add_action_simple('about', self.on_about_activate)
-		self.add_action_simple('quit', self.on_quit)
+			self.add_action_simple('report_bug', self.on_report, None)
+		self.add_action_simple('shortcuts', self.on_shortcuts, \
+		                                         ['<Ctrl>question', '<Ctrl>F1'])
+		self.add_action_simple('help', self.on_help_index, ['F1'])
+		self.add_action_simple('help_main', self.on_help_main, None)
+		self.add_action_simple('help_tools', self.on_help_tools, None)
+		self.add_action_simple('help_canvas', self.on_help_canvas, None)
+		self.add_action_simple('help_selection', self.on_help_selection, None)
+		self.add_action_simple('about', self.on_about, ['<Shift>F1'])
+		self.add_action_simple('quit', self.on_quit, ['<Ctrl>q'])
+		self.add_action_enum('active-window', 0, self.change_active_win)
 
-	def add_accels(self):
-		"""Set all keyboard shortcuts."""
-		self.set_accels_for_action('app.new_window', ['<Ctrl>n'])
-		self.set_accels_for_action('app.help', ['F1'])
-		self.set_accels_for_action('app.quit', ['<Ctrl>q'])
+	# WINDOWS AND CLI MANAGEMENT ###############################################
 
-		self.set_accels_for_action('win.main_color', ['<Ctrl>l'])
-		self.set_accels_for_action('win.secondary_color', ['<Ctrl>r'])
-		self.set_accels_for_action('win.exchange_color', ['<Ctrl>e'])
+	def change_active_win(self, *args):
+		win_id = args[1].get_uint32()
+		for window in self.get_windows():
+			if not isinstance(window, Gtk.ApplicationWindow):
+				continue
+			elif window.get_id() is args[1].get_uint32():
+				window.present()
+		args[0].set_state( GLib.Variant('u', win_id) )
 
-		self.set_accels_for_action('win.go_up', ['<Ctrl>Up'])
-		self.set_accels_for_action('win.go_down', ['<Ctrl>Down'])
-		self.set_accels_for_action('win.go_left', ['<Ctrl>Left'])
-		self.set_accels_for_action('win.go_right', ['<Ctrl>Right'])
+	def update_windows_menu_section(self, *args):
+		# XXX trop appelée à mon goût
+		section = self.get_menubar().get_item_link(6, \
+		          Gio.MENU_LINK_SUBMENU).get_item_link(1, Gio.MENU_LINK_SECTION)
+		section.remove_all()
+		for window in self.get_windows():
+			if not isinstance(window, Gtk.ApplicationWindow):
+				continue
+			elif window.get_title() is None:
+				continue
+			else:
+				detailed_name = 'app.active-window(uint32 ' + \
+				                                      str(window.get_id()) + ')'
+				section.append(window.get_title(), detailed_name)
 
-		self.set_accels_for_action('win.import', ['<Ctrl>i'])
-		self.set_accels_for_action('win.paste', ['<Ctrl>v'])
-		self.set_accels_for_action('win.select_all', ['<Ctrl>a'])
-		self.set_accels_for_action('win.back_to_former_tool', ['<Ctrl>b'])
-
-		self.set_accels_for_action('win.selection_unselect', ['<Ctrl>u'])
-		self.set_accels_for_action('win.selection_cut', ['<Ctrl>x'])
-		self.set_accels_for_action('win.selection_copy', ['<Ctrl>c'])
-		self.set_accels_for_action('win.selection_delete', ['Delete'])
-
-		self.set_accels_for_action('win.show_labels', ['F9'])
-		self.set_accels_for_action('win.main_menu', ['F10'])
-		self.set_accels_for_action('win.options_menu', ['<Shift>F10'])
-		self.set_accels_for_action('win.toggle_preview', ['<Ctrl>m'])
-
-		self.set_accels_for_action('win.new_tab', ['<Ctrl>t'])
-		self.set_accels_for_action('win.close_tab', ['<Ctrl>w'])
-		self.set_accels_for_action('win.open', ['<Ctrl>o'])
-		self.set_accels_for_action('win.save', ['<Ctrl>s'])
-		self.set_accels_for_action('win.save_as', ['<Ctrl><Shift>s'])
-
-		self.set_accels_for_action('win.undo', ['<Ctrl>z'])
-		self.set_accels_for_action('win.redo', ['<Ctrl><Shift>z'])
-
-########
-
-	def open_window_with_file(self, gfile):
-		"""Open a new window with an optional Gio.File as an argument."""
+	def open_window_with_content(self, gfile, get_cb):
+		"""Open a new window with an optional Gio.File as an argument. If get_cb
+		is true, the Gio.File is ignored and the picture is built from the
+		clipboard content."""
 		win = DrawingWindow(application=self)
 		win.present()
-		win.init_window_content(gfile) # this optimization has no effect because
-		# of GLib unknown magic, but should be kept anyway because the window is
-		# presented to the user, making any issue in `init_window_content` very
-		# explicit, and likely to be reported.
+		win.init_window_content(gfile, get_cb) # this optimization has no effect
+		# because of GLib unknown magic, but should be kept anyway because the
+		# window is presented to the user regarless of loading errors, making
+		# any issue in `init_window_content` very explicit, and more likely to
+		# be reported.
+		action = self.lookup_action('active-window')
+		action.set_state( GLib.Variant('u', win.get_id()) )
 		return win
 
 	def on_activate(self, *args):
@@ -168,7 +146,7 @@ class Application(Gtk.Application):
 		it's called by on_cli anyway."""
 		win = self.props.active_window
 		if not win:
-			self.on_new_window_activate()
+			self.on_new_window()
 		else:
 			win.present()
 
@@ -184,40 +162,156 @@ class Application(Gtk.Application):
 
 		if options.contains('version'):
 			print(_("Drawing") + ' ' + self.version)
-			self.on_about_activate()
+			self.on_about()
+		elif options.contains('edit-clipboard'):
+			self.open_window_with_content(None, True)
 
 		# If no file given as argument
 		elif options.contains('new-window') and len(arguments) == 1:
-			self.on_new_window_activate()
+			self.on_new_window()
 		elif options.contains('new-tab') and len(arguments) == 1:
 			win = self.props.active_window
 			if not win:
-				self.on_new_window_activate()
+				self.on_new_window()
 			else:
 				win.present()
-				self.props.active_window.build_new_tab(None)
+				self.props.active_window.build_new_tab(None, None)
 		elif len(arguments) == 1:
 			self.on_activate()
 
 		elif options.contains('new-window'):
-			self.on_new_window_activate()
+			self.on_new_window()
 			for path in arguments:
 				f = self.get_valid_file(args[1], path)
 				if f is not None:
-					self.open_window_with_file(f)
+					self.open_window_with_content(f, False)
 		else: # giving files without '-n' is equivalent to giving files with '-t'
 			for path in arguments:
 				f = self.get_valid_file(args[1], path)
 				if f is not None:
 					win = self.props.active_window
 					if not win:
-						self.open_window_with_file(f)
+						self.open_window_with_content(f, False)
 					else:
 						win.present()
-						self.props.active_window.build_new_tab(f)
+						self.props.active_window.build_new_tab(f, None)
 		# I don't even know if i should return something
 		return 0
 
+	# ACTIONS CALLBACKS ########################################################
+
+	def on_new_window(self, *args):
+		"""Action callback, opening a new window with an empty canvas."""
+		return self.open_window_with_content(None, False)
+
+	def on_report(self, *args):
+		"""Action callback, opening a new issue on the github repo."""
+		win = self.props.active_window
+		Gtk.show_uri_on_window(win, self.git_url + '/issues/new', Gdk.CURRENT_TIME)
+
+	def on_shortcuts(self, *args):
+		"""Action callback, showing the "shortcuts" dialog."""
+		if self.shortcuts_window is not None:
+			self.shortcuts_window.destroy()
+		builder = Gtk.Builder().new_from_resource( \
+		                        '/com/github/maoschanz/drawing/ui/shortcuts.ui')
+		self.shortcuts_window = builder.get_object('shortcuts')
+		self.shortcuts_window.present()
+
+	def on_prefs(self, *args):
+		"""Action callback, showing the preferences window."""
+		if self.prefs_window is not None:
+			self.prefs_window.destroy()
+		wants_csd = not 'ssd' in self.props.active_window.decorations
+		self.prefs_window = DrawingPrefsWindow(self.is_beta(), wants_csd)
+		self.prefs_window.present()
+
+	def on_help_index(self, *args):
+		"""Action callback, showing the index of user help manual."""
+		self.show_help_page('')
+
+	def on_help_main(self, *args):
+		"""Action callback, showing the 'basic features' page of the user help
+		manual."""
+		self.show_help_page('/main_features')
+
+	def on_help_tools(self, *args):
+		"""Action callback, showing the 'tools' page of the user help manual."""
+		self.show_help_page('/drawing_tools')
+
+	def on_help_canvas(self, *args):
+		"""Action callback, showing the 'canvas and selection tools' page of the
+		user help manual."""
+		self.show_help_page('/canvas_tools')
+
+	def on_help_selection(self, *args):
+		"""Action callback, showing the 'selection' page of the user
+		help manual."""
+		self.show_help_page('/selection_tools')
+
+	def show_help_page(self, suffix):
+		win = self.props.active_window
+		Gtk.show_uri_on_window(win, 'help:drawing' + suffix, Gdk.CURRENT_TIME)
+
+	def on_about(self, *args):
+		"""Action callback, showing the "about" dialog."""
+		if self.about_dialog is not None:
+			self.about_dialog.destroy()
+		self.about_dialog = Gtk.AboutDialog(
+			transient_for=self.props.active_window,
+			copyright='© 2019 Romain F. T.',
+			authors=['Romain F. T.'],
+			# To tranlators: "translate" this by your name, it will be displayed in the "about" dialog
+			translator_credits=_("translator-credits"),
+			artists=['Tobias Bernard'],
+			comments=_("A drawing application for the GNOME desktop."),
+			license_type=Gtk.License.GPL_3_0,
+			logo_icon_name=APP_ID,
+			version=str(self.version),
+			website=self.git_url,
+			website_label=_("Report bugs or ideas"))
+		self.about_dialog.connect('response', self.widget_destroy)
+		self.about_dialog.run()
+
+	def widget_destroy(self, widget, button):
+		widget.destroy()
+
+	def on_quit(self, *args):
+		"""Action callback, quitting the app."""
+		self.quit()
+
+	# UTILITIES ################################################################
+
+	def is_beta(self):
+		"""Tells is the app version is even or odd, odd versions being considered
+		as unstable versions. This affects available options and the style of
+		the headerbar."""
+		version_array = self.version.split('.')
+		if (int(version_array[1]) * 5) % 10 == 5:
+			return True
+		else:
+			return False
+
+	def add_action_simple(self, action_name, callback, shortcuts):
+		action = Gio.SimpleAction.new(action_name, None)
+		action.connect('activate', callback)
+		self.add_action(action)
+		if shortcuts is not None:
+			self.set_accels_for_action('app.' + action_name, shortcuts)
+
+	def add_action_boolean(self, action_name, default, callback):
+		action = Gio.SimpleAction().new_stateful(action_name, None, \
+		                                      GLib.Variant.new_boolean(default))
+		action.connect('change-state', callback)
+		self.add_action(action)
+
+	def add_action_enum(self, action_name, default, callback):
+		# XXX attention, celle-ci fonctionne avec un int, pas une string
+		action = Gio.SimpleAction().new_stateful(action_name, \
+		             GLib.VariantType.new('u'), GLib.Variant('u', default))
+		action.connect('change-state', callback)
+		self.add_action(action)
+
 	def get_valid_file(self, app, path):
 		try:
 			f = app.create_file_for_arg(path)
@@ -227,65 +321,12 @@ class Application(Gtk.Application):
 			else:
 				return None
 		except:
-			# Obviously, do not translate the command line.
-			err = _("""Error opening this file. Did you mean
-flatpak run --file-forwarding {0} @@ {1} @@
-?""")
-			print(err.format(APP_ID, path))
+			err = _("Error opening this file. Did you mean %s ?")
+			command = "\n\tflatpak run --file-forwarding {0} @@ {1} @@\n"
+			command = command.format(APP_ID, path)
+			print(err % command)
 			return None
 
-########
-
-	def on_new_window_activate(self, *args):
-		"""Action callback, opening a new window with an empty canvas."""
-		return self.open_window_with_file(None)
-
-	def on_report_activate(self, *args):
-		"""Action callback, opening a new issue on the github repo."""
-		win = self.props.active_window
-		Gtk.show_uri_on_window(win, self.git_url + '/issues/new', Gdk.CURRENT_TIME)
-
-	def on_shortcuts_activate(self, *args):
-		"""Action callback, showing the "shortcuts" dialog."""
-		if self.shortcuts_window is not None:
-			self.shortcuts_window.destroy()
-		builder = Gtk.Builder().new_from_resource( \
-		                        '/com/github/maoschanz/drawing/ui/shortcuts.ui')
-		self.shortcuts_window = builder.get_object('shortcuts')
-		self.shortcuts_window.present()
-
-	def on_prefs_activate(self, *args):
-		"""Action callback, showing the preferences window."""
-		if self.prefs_window is not None:
-			self.prefs_window.destroy()
-		wants_csd = not 'ssd' in self.props.active_window.decorations
-		self.prefs_window = DrawingPrefsWindow(self.is_beta(), wants_csd)
-		self.prefs_window.present()
-
-	def on_help_activate(self, *args):
-		"""Action callback, showing the user help."""
-		win = self.props.active_window
-		Gtk.show_uri_on_window(win, 'help:drawing', Gdk.CURRENT_TIME)
-
-	def on_about_activate(self, *args):
-		"""Action callback, showing the "about" dialog."""
-		if self.about_dialog is not None:
-			self.about_dialog.destroy()
-		self.about_dialog = Gtk.AboutDialog.new()
-		self.about_dialog.set_copyright('© 2019 Romain F. T.')
-		self.about_dialog.set_authors(['Romain F. T.'])
-		# To tranlators: "translate" this by your name, it will be displayed in the "about" dialog
-		self.about_dialog.set_translator_credits(_("translator-credits"))
-		self.about_dialog.set_artists(['Dmitry Z.'])
-		self.about_dialog.set_comments(_("A drawing application for the GNOME desktop."))
-		self.about_dialog.set_license_type(Gtk.License.GPL_3_0)
-		self.about_dialog.set_logo_icon_name(APP_ID)
-		self.about_dialog.set_version(str(self.version))
-		self.about_dialog.set_website(self.git_url)
-		self.about_dialog.set_website_label(_("Report bugs or ideas"))
-		self.about_dialog.run()
-
-	def on_quit(self, *args):
-		"""Action callback, quitting the app."""
-		self.quit()
+	############################################################################
+################################################################################
 
